using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Threading;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using AddiData.ADDIControls;
using AddiData.ADDISocket;
using AddiData;

namespace AutoRefresh
{
    public partial class AutoRefresh : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXExxxx_WebReference.MSXExxxx MSXExxxx;
        private MXCommon_WebReference.DefaultResponse g_MXResponse;
        private MSXExxxx_WebReference.DefaultResponse g_Response;
        private CheckBox[] g_AutoRefreshChannels = new CheckBox[16];
        private Label[] g_ChannelLabels = new Label[16];

        /// <summary>
        /// Store the type of the system
        /// </summary>
        private string systemType;

        /// <summary>
        /// True if the system is a MSX-E3311
        /// </summary>
        private bool isMsxe3311;

        /// <summary>
        /// Store the type of the channels
        /// 0: not available
        /// 1: temperature channel
        /// 2: pressure channel
        /// 3: transducer channel
        /// 4: analog input channel
        /// 5: analog input ICP channel
        /// 6: digital I/O port
        /// 7: incremental counter channel
        /// </summary>
        private int[] channelType;

        /// <summary>
        /// Store the possible trigger mask (or source)
        /// </summary>
        ArrayList possibleTriggerMask;

        private string[] g_ChannelUnity = new string[16];
        private CheckBox[] g_DataFrame = new CheckBox[5];
        ArrayList g_alAcquisitionChannels;
        ArrayList g_alAcquisitionChannelsListe;
        ulong g_ulChannelMask = 0;
        Socket SocketClient = null;
        private int g_iDataStreamSize = 0;
        AsyncClientStream g_Client = null;

        /// <summary>
        /// Text displayed in the value box
        /// </summary>
        String displayedText = string.Empty;

        private int RetValue = 0;
        private int g_syserrno = 0;
        private bool g_boAutoRefreshInit = false;

        public static String ConvertFromTime_T(byte[] bBuffer, int iIndex)
        {
            ulong ulTimeStampHigh = BitConverter.ToUInt32(bBuffer, iIndex);
            ulong ulTimeStampLow = BitConverter.ToUInt32(bBuffer, iIndex + 4);

            return (ConvertFromTime_T(ulTimeStampHigh, ulTimeStampLow));
        }

        public static DateTime ConvertFromTime_T(long time_t)
        {
            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (time_t * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return netTimeT;
        }

        public static String ConvertFromTime_T(ulong ulTimeStampHigh, ulong ulTimeStampLow)
        {
            ulong ulTimeStampLowms = (ulong)(ulTimeStampLow / 1000);
            ulong ulTimeStampLowmus = (ulong)(ulTimeStampLow % 1000);

            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (long)(ulTimeStampHigh * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return (netTimeT.ToString() + "." + ulTimeStampLowms.ToString("D3") + "." + ulTimeStampLowmus.ToString("D3"));
        }


        public AutoRefresh()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Create the list of possible trigger mask, and update the combo box
        /// </summary>
        /// <param name="hasCompare"></param>
        /// <param name="hasIndex"></param>
        private void CreatePossibleTriggerMask(bool hasCompare, bool hasIndex)
        {
            LB_TriggerMask.DataSource = null;

            // Trigger source list box initialisation
            possibleTriggerMask = new ArrayList();
            possibleTriggerMask.Add(new ListData("Disabled", 0));
            possibleTriggerMask.Add(new ListData("Hardware trigger", 1));
            possibleTriggerMask.Add(new ListData("Synchro trigger", 2));

            if (hasCompare == true)
                possibleTriggerMask.Add(new ListData("Compare trigger", 4));

            if (hasIndex == true)
                possibleTriggerMask.Add(new ListData("Index trigger", 8));

            LB_TriggerMask.DisplayMember = "Item";
            LB_TriggerMask.ValueMember = "Value";
            LB_TriggerMask.DataSource = possibleTriggerMask;
        }

        private void AutoRefresh_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
            
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXExxxx = new MSXExxxx_WebReference.MSXExxxx();
            MX_Common.Timeout = 5000;
            MSXExxxx.Timeout = 5000;

            // Refresh time unit list box initialisation
            ArrayList RefreshTimeUnits = new ArrayList();
            RefreshTimeUnits.Add(new ListData("us", 0));
            RefreshTimeUnits.Add(new ListData("ms", 1));
            RefreshTimeUnits.Add(new ListData("s", 2));

            LB_RefreshTimeUnit.DisplayMember = "Item";
            LB_RefreshTimeUnit.ValueMember = "Value";
            LB_RefreshTimeUnit.DataSource = RefreshTimeUnits;

            this.CreatePossibleTriggerMask(false, false);

            // Trigger mode list box initialisation
            ArrayList TriggerModes = new ArrayList();
            TriggerModes.Add(new ListData("One shot trigger", 1));
            TriggerModes.Add(new ListData("Sequence trigger", 2));

            LB_TriggerMode.DisplayMember = "Item";
            LB_TriggerMode.ValueMember = "Value";
            LB_TriggerMode.DataSource = TriggerModes;


            // HW Trigger edge list box initialisation
            ArrayList HWTriggerEdges = new ArrayList();
            HWTriggerEdges.Add(new ListData("Rising edge", 1));
            HWTriggerEdges.Add(new ListData("Falling edge", 2));
            HWTriggerEdges.Add(new ListData("Both front", 3));

            LB_HWEdge.DisplayMember = "Item";
            LB_HWEdge.ValueMember = "Value";
            LB_HWEdge.DataSource = HWTriggerEdges;

            // Save the auto refresh channels check box
            g_AutoRefreshChannels[0] = CB_Channel0;
            g_AutoRefreshChannels[1] = CB_Channel1;
            g_AutoRefreshChannels[2] = CB_Channel2;
            g_AutoRefreshChannels[3] = CB_Channel3;
            g_AutoRefreshChannels[4] = CB_Channel4;
            g_AutoRefreshChannels[5] = CB_Channel5;
            g_AutoRefreshChannels[6] = CB_Channel6;
            g_AutoRefreshChannels[7] = CB_Channel7;
            g_AutoRefreshChannels[8] = CB_Channel8;
            g_AutoRefreshChannels[9] = CB_Channel9;
            g_AutoRefreshChannels[10] = CB_Channel10;
            g_AutoRefreshChannels[11] = CB_Channel11;
            g_AutoRefreshChannels[12] = CB_Channel12;
            g_AutoRefreshChannels[13] = CB_Channel13;
            g_AutoRefreshChannels[14] = CB_Channel14;
            g_AutoRefreshChannels[15] = CB_Channel15;

            g_ChannelLabels[0] = L_TypeCh0;
            g_ChannelLabels[1] = L_TypeCh1;
            g_ChannelLabels[2] = L_TypeCh2;
            g_ChannelLabels[3] = L_TypeCh3;
            g_ChannelLabels[4] = L_TypeCh4;
            g_ChannelLabels[5] = L_TypeCh5;
            g_ChannelLabels[6] = L_TypeCh6;
            g_ChannelLabels[7] = L_TypeCh7;
            g_ChannelLabels[8] = L_TypeCh8;
            g_ChannelLabels[9] = L_TypeCh9;
            g_ChannelLabels[10] = L_TypeCh10;
            g_ChannelLabels[11] = L_TypeCh11;
            g_ChannelLabels[12] = L_TypeCh12;
            g_ChannelLabels[13] = L_TypeCh13;
            g_ChannelLabels[14] = L_TypeCh14;
            g_ChannelLabels[15] = L_TypeCh15;            

            // Save the data frame check box
            g_DataFrame[0] = CB_DataFrameAbsoluteTimeStamp;
            g_DataFrame[1] = CB_DataFrameRelativeTimeStamp;
            g_DataFrame[2] = CB_DataFrameAutoRefreshCounter;
            g_DataFrame[3] = CB_DataFrameStatus;
            g_DataFrame[4] = CB_DataFrameAnalog;

        }

        /// <summary>
        /// User clicks on the connect button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];
            ulong channelCount = 0;
            ulong ulType = 0;
            ulong ulHWPosition = 0;
            ulong ulChannelIndex = 0;
            ulong ulAnaInptChannel = 0;
            ulong []ulAnaInptType;
            ulong []ulAnaInptICP;
            IPAddress address;
            bool hasCompare = false;
            bool hasIndex = false;

            string [] s_Type = {"Not available", "Temperature", "Pressure", "Transducer", "Analog input", "Analog input ICP", "Digital I/O port", "Incremental counter", "Sine/Cosine"};


            if (IPAddress.TryParse(this.ipTextBox.Text, out address) == false)
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            B_StartSample.Enabled = false;
            MX_Common.Url = "http://" + this.ipTextBox.Text + ":5555";
            MSXExxxx.Url = MX_Common.Url;

            // Get the module type
            try
            {
                g_MXResponse = MX_Common.GetModuleType(out pcName);
                if (g_MXResponse.iReturnValue != 0)
                {
                    MessageBox.Show("GetModuleType Error : " + g_MXResponse.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("GetModuleType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // Convert the system type into a string
            this.systemType = enc.GetString(pcName);
            if (systemType.Contains("MX331X{"))
            {
                this.isMsxe3311 = true;
            }
            else
            {
                this.isMsxe3311 = false;
            }

            // Get the number of acquisition channels
            try
            {
                g_Response = MSXExxxx.AcquisitionGetNumberOfChannels(0, out channelCount);
                if (g_Response.iReturnValue != 0)
                {
                    MessageBox.Show("AcquisitionGetNumberOfChannels Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("AcquisitionGetNumberOfChannels Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // Get information on the channels
            this.channelType = new int[channelCount];
            for (ulong i = 0; i < channelCount; i++)
            {
                g_AutoRefreshChannels[i].Enabled = true;
                g_AutoRefreshChannels[i].Checked = true;
                try
                {
                    g_Response = MSXExxxx.AcquisitionGetChannelInfo(i,0,out ulType, out ulHWPosition,out ulChannelIndex);
                    if (g_Response.iReturnValue != 0)
                    {
                        MessageBox.Show("AcquisitionGetChannelInfo Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                    g_ChannelLabels[i].Text = s_Type[ulType];
                    channelType[i] = (int)ulType;

                    // Check if incremental counter or sin cos
                    if ((ulType == 7) || (ulType == 8))
                        hasCompare = true;

                    // Check if sin cos
                    if (ulType == 8)
                        hasIndex = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("AcquisitionGetChannelInfo Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }

            for (ulong i = 0; i < channelCount; i++)
            {
                switch (channelType[i])
                {
                    case 1:
                        g_ChannelUnity[i] = "C";
                        break;

                    case 3:
                        g_ChannelUnity[i] = "mm";
                        break;

                    case 4:
                    case 5:
                        try
                        {
                            g_Response = MSXExxxx.AnalogInputGetChannelsType(0, out ulAnaInptType, out ulAnaInptICP);
                            if (g_Response.iReturnValue != 0)
                            {
                                MessageBox.Show("AnalogInputGetChannelsType Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                return;
                            }

                            if (ulAnaInptType[ulAnaInptChannel] == 0)
                            {
                                g_ChannelUnity[i] = "V";
                            }
                            else
                            {
                                g_ChannelUnity[i] = "A";
                            }
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show("AnalogInputGetChannelsType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            return;
                        }
                        ulAnaInptChannel++;
                        break;

                    default:
                        g_ChannelUnity[i] = "";
                        break;
                }
            }

            for (ulong i = channelCount; i < 16; i++)
            {
                g_AutoRefreshChannels[i].Enabled = false;
                g_AutoRefreshChannels[i].Checked = false;
                g_ChannelLabels[i].Text = "";
            }

            // Update the list of possible triggers
            CreatePossibleTriggerMask(hasCompare, hasIndex);

            B_StartSample.Enabled = true;
            GB_AutoRefresh.Enabled = true;
            GB_DataFormat.Enabled = true;
            GB_TriggerConfiguration.Enabled = true;
            B_StartSample.Enabled = true;
            GB_AcquisitionMode.Enabled = true;
        }

        private bool boCheckConfiguration(out ulong ulChannelMask, out ulong ulDataFormat, out int iDataStreamSize)
        {
            ulong ulChannels = 0;

            ulChannelMask = 0;
            ulDataFormat = 0;
            iDataStreamSize = 0;

            // Check the channel mask
            g_alAcquisitionChannels = new ArrayList();
            g_alAcquisitionChannelsListe = new ArrayList();
            for (int i = 0; i < 16; i++)
            {
                ulChannelMask = ulChannelMask | (Convert.ToUInt32(g_AutoRefreshChannels[i].Checked) << i);
                ulChannels = ulChannels + Convert.ToUInt32(g_AutoRefreshChannels[i].Checked);

                if (g_AutoRefreshChannels[i].Checked)
                {
                    g_alAcquisitionChannels.Add("Channel " + i.ToString("D2") + ":");
                    g_alAcquisitionChannelsListe.Add(i);
                }
            }

            if (ulChannelMask == 0)
            {
                MessageBox.Show("No channel selected", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the average value
            if ((Convert.ToInt32(TB_AverageValue.Text) < 1) || (Convert.ToInt32(TB_AverageValue.Text) > 255))
            {
                MessageBox.Show("Average value selection error (0 < value < 256)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the hardware trigger sequence number
            if ((TB_SequenceTrigger.Enabled == true) && ((Convert.ToInt64(TB_SequenceTrigger.Text) < 1) || (Convert.ToInt64(TB_SequenceTrigger.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Selection of sequence number to acquire for each trigger is wrong (0 < value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }


            // Get the data frame format
            for (int i = 0; i < g_DataFrame.Length; i++)
            {
                ulDataFormat = ulDataFormat | (Convert.ToUInt32(g_DataFrame[i].Checked) << i);
            }

            // Get the data strem size
            iDataStreamSize = (int)((ulChannels * 4) +
                                    ((Convert.ToUInt32(CB_DataFrameAbsoluteTimeStamp.Checked)|(Convert.ToUInt32(CB_DataFrameRelativeTimeStamp.Checked))) * 8) +
                                    (Convert.ToUInt32(CB_DataFrameAutoRefreshCounter.Checked) * 4) +
                                    (Convert.ToUInt32(CB_DataFrameStatus.Checked) * 4));

            return (true);
        }

        /// <summary>
        /// Client thread
        /// </summary>
        private void ClientThreadFct()
        {
            String sOutString = string.Empty;
            int SocketDataIndex;
            byte[] SocketData = new byte[g_Client.SingleFramesSize];

            while ((SocketData = g_Client.ReadData()) != null)
            {
                SocketDataIndex = 0;

                while (SocketData.Length != SocketDataIndex)
                {
                    sOutString = "";

                    // Receive the time stamp?
                    if (CB_DataFrameAbsoluteTimeStamp.Checked == true)
                    {
                        sOutString = "Time stamp: " + ConvertFromTime_T(SocketData, SocketDataIndex) + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 8;
                    }
                    if (CB_DataFrameRelativeTimeStamp.Checked == true)
                    {
                        sOutString = "Time stamp: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + ":" + BitConverter.ToUInt32(SocketData, SocketDataIndex + 4).ToString("D6") + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 8;
                    }

                    // Receive the auto-refresh counter ?
                    if (CB_DataFrameAutoRefreshCounter.Checked == true)
                    {
                        sOutString += "Auto refresh counter: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 4;
                    }

                    // Receive the status ?
                    if (CB_DataFrameStatus.Checked == true)
                    {
                        sOutString += "Status: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 4;
                    }

                    // Get the values of the channels of the acquisition
                    int iChannelsCpt = 0;
                    foreach (String Channel in g_alAcquisitionChannels)
                    {
                        // Test if analog value
                        if (CB_DataFrameAnalog.Checked == true)
                        {
                            switch (this.channelType[(int)g_alAcquisitionChannelsListe[iChannelsCpt]])
                            {
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                case 4:
                                case 5:
                                case 8:
                                    // 0 -> 5, 8 support float conversion
                                    sOutString += Channel + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString() + g_ChannelUnity[(int)g_alAcquisitionChannelsListe[iChannelsCpt]];
                                    break;
                                case 6:
                                case 7:
                                    // 6 (digital I/O) and 7 (incremental counter) do not supports float conversion
                                    sOutString += Channel + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X8") + g_ChannelUnity[(int)g_alAcquisitionChannelsListe[iChannelsCpt]];
                                    break;
                            }
                        }
                        else
                        {
                            // MSX-E3311: digital value, other MSX-E float value
                            if ((this.channelType[(int)g_alAcquisitionChannelsListe[iChannelsCpt]] == 2) && (this.isMsxe3311 == false))
                            {
                                sOutString += Channel + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString() + g_ChannelUnity[(int)g_alAcquisitionChannelsListe[iChannelsCpt]];
                            }
                            else
                            {
                                sOutString += Channel + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X8");
                            }
                        }
                        
                        SocketDataIndex = SocketDataIndex + 4;
                        iChannelsCpt++;
                        sOutString += Environment.NewLine;
                    }
                    displayedText = sOutString;
                }
            }
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void B_StartSample_Click(object sender, EventArgs e)
        {
            ulong ulDataFormat;
            Thread ClientThread;

            // Check the user configuration
            if (boCheckConfiguration(out g_ulChannelMask, out ulDataFormat, out g_iDataStreamSize) == true)
            {
                try
                {
                    SocketClient = null;
                    ClientThread = null;

                    // Test if receive data via the data server
                    if (RB_DataServer.Checked == true)
                    {
                        /* Create the client stream */
                        g_Client = new AddiData.ADDISocket.AsyncClientStream();

                        // DATA FORMAT CONFIGURATION
                        g_Client.SingleFramesSize = g_iDataStreamSize;
                        g_Client.NbrOfFrames = 1;


                        // Open the socket connection to receive the data
                        SocketClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                        SocketClient.Connect(this.ipTextBox.Text, 8989);
                        SocketClient.ReceiveBufferSize = 1048576;

                        // Start the first read socket 
                        // Start the read socket and the user thread
                        g_Client.Start(SocketClient);
                        ClientThread = new Thread(ClientThreadFct);
                    }

                    displayedText = "";


                        try
                        {
                            //Init and start the auto refresh acquisition
                            RetValue = MSXExxxx.AcquisitionAutoRefreshInitAndStart(g_ulChannelMask,
                                                                                  Convert.ToUInt64(TB_AverageValue.Text),
                                                                                  Convert.ToUInt64(TB_RefreshTime.Text),
                                                                                  Convert.ToUInt64(LB_RefreshTimeUnit.SelectedIndex),
                                                                                  Convert.ToUInt64(LB_TriggerMask.SelectedValue),
                                                                                  Convert.ToUInt64(LB_TriggerMode.SelectedValue),
                                                                                  Convert.ToUInt64(LB_HWEdge.SelectedValue),
                                                                                  Convert.ToUInt64(TB_HWTriggerCount.Text),
                                                                                  Convert.ToUInt64(TB_SequenceTrigger.Text),
                                                                                  ulDataFormat,
                                                                                  Convert.ToUInt64(CB_ForceStart.CheckState),
                                                                                  0, 0, 0, out g_syserrno);


                            if (RetValue != 0)
                            {
                                MessageBox.Show("AcquisitionAutoRefreshInitAndStart Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                            else
                            {
                                // Test if receive data via the data server
                                if (RB_DataServer.Checked == true)
                                {
                                    ClientThread.Priority = ThreadPriority.Lowest;
                                    ClientThread.Start();
                                }
                                else
                                {
                                    ReadTimer.Enabled = true;
                                }

                                DisplayTimer.Enabled = true;
                                g_boAutoRefreshInit = true;
                                B_Connect.Enabled = false;
                                B_StartSample.Enabled = false;
                                GB_AutoRefresh.Enabled = false;
                                GB_DataFormat.Enabled = false;
                                GB_TriggerConfiguration.Enabled = false;
                                B_StopSample.Enabled = true;
                                GB_AcquisitionMode.Enabled = false;
                            }
                        }

                        catch (Exception ex)
                        {
                            MessageBox.Show("AcquisitionAutoRefreshInitAndStart Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

                // Test if error occur
                if (g_boAutoRefreshInit == false)
                {
                    B_StopSample_Click(sender, e);
                }
            }

        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void B_StopSample_Click(object sender, EventArgs e)
        {
            DisplayTimer.Enabled = false;
            ReadTimer.Enabled = false;

            B_StartSample.Enabled = !B_StopSample.Enabled;
            B_StopSample.Enabled = false;

            // Test if socket open
            if (SocketClient != null)
            {
                if (g_Client != null)
                    g_Client.Stoop();

                try
                {
                    // Close the socket connection
                    SocketClient.Disconnect(false);
                    SocketClient.Close();
                    SocketClient = null;
                    System.GC.Collect();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket close connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            // Test if auto refresh started
            if (g_boAutoRefreshInit == true)
            {
                g_boAutoRefreshInit = false;
                B_Connect.Enabled = true;
                B_StartSample.Enabled = true;
                GB_AutoRefresh.Enabled = true;
                GB_DataFormat.Enabled = true;
                GB_TriggerConfiguration.Enabled = true;
                GB_AcquisitionMode.Enabled = true;

                try
                {

                    RetValue = MSXExxxx.AcquisitionAutoRefreshStopAndRelease(out g_syserrno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("AcquisitionAutoRefreshStopAndRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("AcquisitionAutoRefreshStopAndRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            TB_AutoRefreshInfo.Text = displayedText;
        }

        /// <summary>
        /// Tick of the read timer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReadTimer_Tick(object sender, EventArgs e)
        {
            double [] values;
            String outString = string.Empty;
            int Cpt = 0;
            int iChannelsCpt = 0;
            ulong TestChannel = 0;
            ulong tus = 0;
            ulong ts = 0;
            ulong milliseconds = 0;
            ulong microseconds = 0;
            ulong counter = 0;

            // Using SOAP, read the auto-refresh values
            try
            {
                // Read the auto refresh value
                MSXExxxx_WebReference.DefaultResponse response = MSXExxxx.AcquisitionAutoRefreshGetValues(Convert.ToUInt64(CB_Blocking.CheckState), out tus, out ts, out counter, out values);
                
                // Test if error occur
                if (response.iReturnValue != 0)
                {
                    // Special case
                    if (response.iReturnValue == -3)
                    {
                        displayedText = "Timeout of 2 seconds, while waiting the data (blocking mode enabled). Still waiting a data...";
                        return;
                    }
                    else if (response.iReturnValue == -4)
                    {
                        displayedText = "Timeout of 2 seconds, while waiting the first data (blocking mode disabled). Still waiting the first data...";
                        return;
                    }
                    else
                    {
                        ReadTimer.Enabled = false;
                        B_StopSample_Click(sender, null);
                        MessageBox.Show("AcquisitionAutoRefreshGetValues Error : " + response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                }
            }
            catch (Exception ex)
            {
                ReadTimer.Enabled = false;
                B_StopSample_Click(sender, null);
                MessageBox.Show("AcquisitionAutoRefreshGetValues Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // From the tus, extract the milli seconds and the micro seconds part (useful to display the time stamp with a better precision)
            microseconds = (ulong)(tus / 1000);
            milliseconds = (ulong)(tus % 1000);
            outString = "Time stamp: " + ConvertFromTime_T((long)ts).ToString() + "." + microseconds.ToString("D3") + "." + milliseconds.ToString("D3") + Environment.NewLine;
            outString += "Auto refresh counter: " + counter.ToString() + Environment.NewLine;

            // Get the auto refresh channel mask
            for (Cpt = 0, TestChannel = 1, iChannelsCpt = 0; Cpt < 16; Cpt++, TestChannel = TestChannel * 2)
            {
                // Test if channel used
                if ((g_ulChannelMask & TestChannel) != 0)
                {
                    // Test if user selected analog value
                    if (CB_DataFrameAnalog.Checked == true)
                    {
                        switch (this.channelType[Cpt])
                        {
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 8:
                                // 0 -> 5, 8 support float conversion
                                outString += g_alAcquisitionChannels[iChannelsCpt] + values[Cpt].ToString("F5") + g_ChannelUnity[Cpt] + Environment.NewLine;
                                break;
                            case 6:
                            case 7:
                                // 6 (digital I/O) and 7 (incremental counter) do not supports float conversion
                                outString += g_alAcquisitionChannels[iChannelsCpt] + Convert.ToInt64(values[Cpt]).ToString("X8") + Environment.NewLine;
                                break;
                        }
                    }
                    // User selected digital value
                    else
                    {
                        // MSX-E3311: digital value, other MSX-E float value
                        if ((this.channelType[Cpt] == 2) && (this.isMsxe3311 == false))
                        {
                            outString += g_alAcquisitionChannels[iChannelsCpt] + values[Cpt].ToString("F5") + g_ChannelUnity[Cpt] + Environment.NewLine;
                        }
                        else
                        {
                            outString += g_alAcquisitionChannels[iChannelsCpt] + Convert.ToInt64(values[Cpt]).ToString("X8") + Environment.NewLine;
                        }                            
                    }

                    iChannelsCpt++;
                }
            }
            displayedText = outString;
        }

        /// <summary>
        /// Form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AutoRefresh_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();

            B_StopSample_Click(sender, null);
        }

        private void TB_KeyPress_OnlyNumber(object sender, KeyPressEventArgs e)
        {
            if (!char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar))
                e.Handled = true;
        }

        private void RB_SOAP_CheckedChanged(object sender, EventArgs e)
        {
            CB_Blocking.Enabled = true;
            CB_DataFrameAbsoluteTimeStamp.Enabled = false;
            CB_DataFrameRelativeTimeStamp.Enabled = false;
            CB_DataFrameAutoRefreshCounter.Enabled = false;
            CB_DataFrameStatus.Enabled = false;
        }

        private void RB_DataServer_CheckedChanged(object sender, EventArgs e)
        {
            CB_Blocking.Enabled = false;
            CB_DataFrameAbsoluteTimeStamp.Enabled = true;
            CB_DataFrameRelativeTimeStamp.Enabled = true;
            CB_DataFrameAutoRefreshCounter.Enabled = true;
            CB_DataFrameStatus.Enabled = true;
        }

        /// <summary>
        /// Select (or deselect) absolute time stamp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CB_DataFrameAbsoluteTimeStamp_CheckedChanged(object sender, EventArgs e)
        {
            if (CB_DataFrameAbsoluteTimeStamp.Checked == true)
            {
                CB_DataFrameRelativeTimeStamp.Checked = false;
            }
        }

        /// <summary>
        /// Select (or deselect) relative time stamp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CB_DataFrameRelativeTimeStamp_CheckedChanged(object sender, EventArgs e)
        {
            if (CB_DataFrameRelativeTimeStamp.Checked == true)
            {
                CB_DataFrameAbsoluteTimeStamp.Checked = false;
            }
        }

        /// <summary>
        /// Update the control
        /// </summary>
        private void UpdateTriggerControl()
        {
            LB_TriggerMode.Enabled = (LB_TriggerMask.SelectedIndex != 0);
            TB_SequenceTrigger.Enabled = (LB_TriggerMask.SelectedIndex != 0) && (LB_TriggerMode.SelectedIndex == 1);
            LB_HWEdge.Enabled = (LB_TriggerMask.SelectedIndex == 1);
            TB_HWTriggerCount.Enabled = (LB_TriggerMask.SelectedIndex == 1);
        }

        /// <summary>
        /// Change the trigger mask
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LB_TriggerMask_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.UpdateTriggerControl();
        }

        /// <summary>
        /// Change the trigger mode
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LB_TriggerMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.UpdateTriggerControl();
        }
    }
}