using System;
using System.Collections.Generic;
using System.Text;

namespace Diagnosis
{
    /// <summary>
    /// Represent a digital I/O port
    /// </summary>
    public class DigitalIOPort
    {
        /// <summary>
        /// Number of channels of the port
        /// </summary>
        private ulong channelNumber;

        /// <summary>
        /// Mask of available inputs
        /// </summary>
        private ulong availableInputs;

        /// <summary>
        /// Mask of available outputs
        /// </summary>
        private ulong availableOutputs;

        /// <summary>
        /// Index of the port
        /// </summary>
        private ulong index;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="index"></param>
        /// <param name="availableInputs"></param>
        /// <param name="availableOutputs"></param>
        public DigitalIOPort(ulong index, ulong availableInputs, ulong availableOutputs)
        {
            this.index = index;
            this.availableInputs = availableInputs;
            this.availableOutputs = availableOutputs;
            this.channelNumber = 0;

            // Get the number of channels of the port
            for (int i = 0; i < 32; i++)
            {
                if ((((this.availableInputs >> i) & 1) == 1)
                    || (((this.availableOutputs >> i) & 1) == 1))
                {
                    this.channelNumber++;
                }
            }
        }

        /// <summary>
        /// Return the index of the port
        /// </summary>
        public ulong Index
        {
            get { return this.index; }
        }

        /// <summary>
        /// Return the available inputs
        /// </summary>
        public ulong AvailableInputs
        {
            get { return this.availableInputs; }
        }

        /// <summary>
        /// Return the available outputs
        /// </summary>
        public ulong AvailableOutputs
        {
            get { return this.availableOutputs; }
        }

        /// <summary>
        /// Return the number of channels
        /// </summary>
        public ulong ChannelNumber
        {
            get { return this.channelNumber; }
        }
    }
}
