using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Net;
using System.Net.Sockets;

namespace Calibration
{
    public partial class Calibration : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXExxxx_WebReference.MSXExxxx MSXExxxx;
        private MSXExxxx_WebReference.DefaultResponse Response;
        private int RetValue = 0;
        private int syserno = 0;
        private UInt64 ulLastStatus = 0;
        double  dRange              = 0;
        double  dPosition           = 0;
        private TranducerInfo[] TranducerInformations;


        public class TranducerInfo
        {
            private ulong ulSelectionIndex;
            private String sName;
            private ulong ulCalibrationStatus;
            private ulong ulType;
            private ulong ulFrequency;
            private ulong ulImpedance;
            private double dVeff;
            private double dSensibility;
            private double dRange;

            public TranducerInfo(ulong SelectionIndex,
                                 String Name,
                                 ulong CalibrationStatus,
                                 ulong Type,
                                 ulong Frequency,
                                 ulong Impedance,
                                 double Veff,
                                 double Sensibility,
                                 double Range)
            {
                ulSelectionIndex = SelectionIndex;
                sName = Name.Remove(Name.IndexOf((char)0));
                ulCalibrationStatus = CalibrationStatus;
                ulType = Type;
                ulFrequency = Frequency;
                ulImpedance = Impedance;
                dVeff = Veff;
                dSensibility = Sensibility;
                dRange = Range;
            }

            public ulong SelectionIndex { get { return (ulSelectionIndex); } set { ulSelectionIndex = value; } }
            public ulong CalibrationStatus { get { return (ulCalibrationStatus); } set { ulCalibrationStatus = value; } }
            public ulong Type { get { return (ulType); } set { ulType = value; } }
            public ulong Frequency { get { return (ulFrequency); } set { ulFrequency = value; } }
            public ulong Impedance { get { return (ulImpedance); } set { ulImpedance = value; } }
            public double Veff { get { return (dVeff); } set { dVeff = value; } }
            public double Sensibility { get { return (dSensibility); } set { dSensibility = value; } }
            public double Range { get { return (dRange); } set { dRange = value; } }
            public String Name { get { return (sName); } set { sName = value; } }

            public override String ToString()
            {
                String sRet;

                sRet = "Transducer Selection Index : " + ulSelectionIndex.ToString();
                sRet = sRet + Environment.NewLine + "Name                                   : " + sName;
                sRet = sRet + Environment.NewLine + "Calibration status                  : " + ulCalibrationStatus.ToString();
                sRet = sRet + Environment.NewLine + "Type Value                          : " + ulType.ToString();
                sRet = sRet + Environment.NewLine + "Frequency                           : " + ulFrequency.ToString() + " Hz";
                sRet = sRet + Environment.NewLine + "Impedance                          : " + ulImpedance.ToString() + " Ohm";
                sRet = sRet + Environment.NewLine + "Nominal Voltage                  : " + dVeff.ToString() + " V";
                sRet = sRet + Environment.NewLine + "Sensibility                            : " + dSensibility.ToString() + " mV/V/mm";
                sRet = sRet + Environment.NewLine + "Range                                 : +/-" + dRange.ToString() + " mm";

                return (sRet);
            }
        }

        public class ListData
        {
            private string myItem;
            private long myValue;

            public ListData(string strItem, long lValue)
            {

                this.myItem = strItem;
                this.myValue = lValue;
            }

            public string Item
            {
                get { return myItem; }
            }

            public long Value
            {

                get { return myValue; }
            }
        }

        public Calibration()
        {
            InitializeComponent();
        }

        private void Calibration_Load(object sender, EventArgs e)
        {
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXExxxx = new MSXExxxx_WebReference.MSXExxxx();
            MX_Common.Timeout = 5000;
            MSXExxxx.Timeout = 5000;
            TB_CalibrationStatus.ReadOnly = true;
            

        }


        private void LB_TransducerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            L_TransducerInfo.Text = TranducerInformations[LB_TransducerList.SelectedIndex].ToString();

        }

        private void B_Connect_Click(object sender, EventArgs e)
        {
            ulong ulNbrOfChannels;
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];

            if ((TB_IPAddress_0.Text.ToString() == "") || (TB_IPAddress_1.Text.ToString() == "") || (TB_IPAddress_2.Text.ToString() == "") || (TB_IPAddress_3.Text.ToString() == ""))
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                if ((Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256))
                {
                    if ((Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256))
                    {
                        if ((Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256))
                        {
                            if ((Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256))
                            {
                                LB_Channel.Enabled = false;
                                B_StartCalibration.Enabled = false;
                                MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
                                MSXExxxx.Url = MX_Common.Url;


                                try
                                {
                                    // Get number of channels
                                    Response = MSXExxxx.TransducerGetNumberOfChannels(0, out ulNbrOfChannels);
                                    if (Response.iReturnValue != 0)
                                    {
                                        MessageBox.Show("TransducerGetNumberOfChannels Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }
                                    // Channel list box initialisation
                                    ArrayList Channels = new ArrayList();

                                    for (long i = 0; i < (long)ulNbrOfChannels; i++)
                                    {
                                        Channels.Add(new ListData(i.ToString(), i));
                                    }

                                    LB_Channel.Enabled = true;
                                    LB_Channel.DisplayMember = "Item";
                                    LB_Channel.ValueMember = "Value";
                                    LB_Channel.DataSource = Channels;

                                    B_StartCalibration.Enabled = true;

                                }

                                catch (Exception ex)
                                {
                                    MessageBox.Show("TransducerGetNumberOfChannels Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }
                            }
                            else
                            {
                                MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                        else
                        {
                            MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private void LB_Channel_SelectedIndexChanged(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            ulong ulTransducerNbr = 1;
            ulong[] ulCalibrated = null;
            MSXExxxx_WebReference.TransducerTypeInfo[] sTransducerTypeInfo = null;
            GB_Transducers.Enabled = false;

            try
            {
                // Get the number of transducers type

                Response = MSXExxxx.TransducerGetChannelTransducersList(Convert.ToUInt32(LB_Channel.SelectedValue), 0, out ulTransducerNbr, out ulCalibrated, out sTransducerTypeInfo);
                if (Response.iReturnValue != 0)
                {
                    MessageBox.Show("TransducerGetChannelTransducersList Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

                TranducerInformations = new TranducerInfo[ulTransducerNbr];
                ArrayList Transducers = new ArrayList();
                L_NbrOfTransducers.Text = ulTransducerNbr.ToString();

                for (ulong ulCpt = 0; ulCpt < ulTransducerNbr; ulCpt++)
                {
                    // Save the transducer informations
                    TranducerInformations[ulCpt] = new TranducerInfo(sTransducerTypeInfo[ulCpt].ulTransducerIndex,
                                                                     enc.GetString(sTransducerTypeInfo[ulCpt].cName),
                                                                     ulCalibrated[ulCpt],
                                                                     sTransducerTypeInfo[ulCpt].ulType,
                                                                     sTransducerTypeInfo[ulCpt].ulNominalFrequency,
                                                                     sTransducerTypeInfo[ulCpt].ulImpedance,
                                                                     sTransducerTypeInfo[ulCpt].dVeff,
                                                                     sTransducerTypeInfo[ulCpt].dSensitivity,
                                                                     sTransducerTypeInfo[ulCpt].dRange);

                    Transducers.Add(new ListData(enc.GetString(sTransducerTypeInfo[ulCpt].cName), (long)sTransducerTypeInfo[ulCpt].ulTransducerIndex));
                }

                LB_TransducerList.DataSource = Transducers;
                LB_TransducerList.DisplayMember = "Item";
                LB_TransducerList.ValueMember = "Value";
                GB_Transducers.Enabled = true;
            }
                
            catch (Exception ex)
            {
                MessageBox.Show("TransducerGetChannelTransducersList Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
        }


        private void B_StartCalibration_Click(object sender, EventArgs e)
        {
            try
            {

                ulLastStatus = 0;
                TB_CalibrationStatus.Text = "";
                L_CalibrationValue.Text = "";

                dRange = TranducerInformations[LB_TransducerList.SelectedIndex].Range;
                dPosition = dRange / 2;

                //Init and start the calibration

                RetValue = MSXExxxx.TransducerCalibrationStart(Convert.ToUInt32(LB_Channel.SelectedValue),
                                                               Convert.ToUInt32(LB_TransducerList.SelectedValue),
                                                               dPosition,
                                                               0,
                                                               out syserno);

                if (RetValue != 0)
                {
                    MessageBox.Show("TransducerCalibrationStart Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    B_Connect.Enabled = false;
                    GB_Transducers.Enabled = false;
                    B_StartCalibration.Enabled = false;
                    B_NextStep.Enabled = false;
                    LB_Channel.Enabled = false;
                    B_BreakCalibration.Enabled = true;
                    T_Calibration.Enabled = true;
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("TransducerCalibrationStart Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void B_NextStep_Click(object sender, EventArgs e)
        {
            B_NextStep.Enabled = false;
            T_Color.Enabled = false;
            B_NextStep.ForeColor = Color.Black;

            try
            {
                //Init and start the calibration

                RetValue = MSXExxxx.TransducerCalibrationNextStep(0, out syserno);


                if (RetValue != 0)
                {
                    MessageBox.Show("TransducerCalibrationNextStep Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("TransducerCalibrationNextStep Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void B_BreakCalibration_Click(object sender, EventArgs e)
        {
            if (B_BreakCalibration.Enabled == true)
            {
                try
                {
                    //Init and start the calibration

                    RetValue = MSXExxxx.TransducerCalibrationBreak(0, out syserno);

                    if (RetValue != 0)
                    {
                        MessageBox.Show("TransducerCalibrationBreak Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("TransducerCalibrationBreak Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }


        private void T_Calibration_Tick(object sender, EventArgs e)
        {
            UInt64 ulStatus          = 0;
            UInt64 ulChannel         = 0;
            UInt64 ulTransducerIndex = 0;
            double dUserPosition     = 0;
            double dCurrentValue     = 0;

            try
            {
                // Get calibration current status
                Response = MSXExxxx.TransducerCalibrationGetCurrentStatus(0, out ulStatus, out ulChannel, out ulTransducerIndex, out dUserPosition, out dCurrentValue);

                // Test if error occur

                if (Response.iReturnValue != 0)
                {
                    MessageBox.Show("TransducerCalibrationGetCurrentStatus Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("TransducerCalibrationGetCurrentStatus Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            // Test if end user position calibration 
            if ((ulStatus == 7) && (ulLastStatus != 7))
            {
                // Stop the calibration 
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "!!!User have break the calibration";
            }

            // Test if primary calibration error occur 
            else if ((ulStatus == 8) && (ulLastStatus != 8))
            {
                // Stop the calibration 
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "Can not calibrate the primary";
            }

            // Test if primary null position error occur 
            else if ((ulStatus == 9) && (ulLastStatus != 9))
            {
                // Stop the calibration 
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "The 0mm position can not by found." + Environment.NewLine + "Restart the calibration and select a new position for the 0mm.";
            }

            // Test if primary null position error occur 
            else if ((ulStatus == 10) && (ulLastStatus != 10))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "The user position can not by found." + Environment.NewLine + "Restart the calibration and select a new position";
            }

            //Test if start primary calibration 
            else if ((ulStatus == 1) && (ulLastStatus == 0))
            {
                TB_CalibrationStatus.Text = "Primary calibration started" + Environment.NewLine;
                L_CalibrationValue.Text = "Primary calibration : " + dCurrentValue.ToString("F4") + "V";
            }

            // Test if primary calibration started 
            else if ((ulStatus == 1) && (ulLastStatus == 1))
            {
                L_CalibrationValue.Text = "Primary calibration : " + dCurrentValue.ToString("F4") + "V";
            }

            // Test if end primary calibration 
            else if ((ulStatus == 2) && (ulLastStatus != 2))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Primary calibration conclude" + Environment.NewLine;
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Set the transducer connected to channel " + ulChannel.ToString() + " to position 0 mm and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }

            // Test if wait user set transducer to 0mm 
            else if ((ulStatus == 2) && (ulLastStatus == 2))
            {
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if start null position calibration 
            else if ((ulStatus == 3) && (ulLastStatus != 3))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Null position calibration started" + Environment.NewLine;
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if null position calibration 
            else if ((ulStatus == 3) && (ulLastStatus == 3))
            {
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if end user position calibration 
            else if ((ulStatus == 4) && (ulLastStatus != 4))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Null position calibration conclude" + Environment.NewLine;
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Set the transducer connected to channel " + ulChannel.ToString() + " to position " + dPosition.ToString("G4") + "mm and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }

            // Test if wait user set transducer to user position 
            else if ((ulStatus == 4) && (ulLastStatus == 4))
            {
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if start user position calibration 
            else if ((ulStatus == 5) && (ulLastStatus != 5))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "User position calibration started" + Environment.NewLine;
            }

            // Test if user position calibration 
            else if ((ulStatus == 5) && (ulLastStatus == 5))
            {
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if end user position calibration 
            if ((ulStatus == 6) && (ulLastStatus != 6))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "User position calibration conclude" + Environment.NewLine;
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "End of calibration" + Environment.NewLine;
                L_CalibrationValue.Text = "Transducer position = " + dCurrentValue.ToString("F4") + " mm";
            }

            // Test if end calibration 
            if (ulStatus >= 6)
            {
                B_StartCalibration.Enabled = true;
                B_BreakCalibration.Enabled = false;
                B_NextStep.Enabled = false;
                LB_Channel.Enabled = true;
                GB_Transducers.Enabled = true;
                T_Calibration.Enabled = false;
                T_Color.Enabled = false;
                B_NextStep.ForeColor = Color.Black;
                B_Connect.Enabled = true;
            }

            if (ulLastStatus != ulStatus)
            {
                TB_CalibrationStatus.SelectionStart = TB_CalibrationStatus.Text.Length;
                TB_CalibrationStatus.ScrollToCaret();
            }

            ulLastStatus = ulStatus;
        }

        private void T_Color_Tick(object sender, EventArgs e)
        {
            if (B_NextStep.ForeColor == B_NextStep.BackColor)
                B_NextStep.ForeColor = Color.Black;
            else
                B_NextStep.ForeColor = B_NextStep.BackColor;
        }

        private void Calibration_FormClosing(object sender, FormClosingEventArgs e)
        {
            B_BreakCalibration_Click(sender, null);
        }



    }
}