using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Configuration
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Link to the soap interface
        /// </summary>
        MSXExxxx.MSXExxxx msxe_soap;

        /// <summary>
        /// Store the nb of channels
        /// </summary>
        ulong channel_nb = 0;

        /// <summary>
        /// Graphics component to represent the channels
        /// </summary>
        Label [] channel_id;
        Label[] type;
        ComboBox[] transducer;
        RadioButton[] invert_value;
        RadioButton[] dont_invert_value;
        Panel[] invert_or_not;
        CheckBox[] calibrated;

        /// <summary>
        /// Contains the transducer data sources
        /// </summary>
        String[][] datasource;
        ulong[][] transducer_indexes;

        /// <summary>
        /// Constructor
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            // Creation of the soap interface
            msxe_soap = new MSXExxxx.MSXExxxx();
        }

        /// <summary>
        /// On load of the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            ip_field_0.Text = Properties.Settings.Default.ip_0;
            ip_field_1.Text = Properties.Settings.Default.ip_1;
            ip_field_2.Text = Properties.Settings.Default.ip_2;
            ip_field_3.Text = Properties.Settings.Default.ip_3;
        }
        
        /// <summary>
        /// When closing the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip_0 = ip_field_0.Text;
            Properties.Settings.Default.ip_1 = ip_field_1.Text;
            Properties.Settings.Default.ip_2 = ip_field_2.Text;
            Properties.Settings.Default.ip_3 = ip_field_3.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Convert the name of the transducer into a string
        /// </summary>
        /// <param name="name"></param>
        private string get_name(byte[] byte_name)
        {
            string name = "";
            int i = 0;
            for (i = 0; i < byte_name.Length; i++)
            {
                if (byte_name[i] == '\0')
                    return name;

                name += (char)byte_name[i];
            }
            return name;
        }

        /// <summary>
        /// Get available transducers
        /// </summary>
        private void get_transducer_information(ulong channel)
        {
            ulong transducer_nb = 0;
            ulong i = 0;
            ulong[] transducer_calibrated = new ulong[16];
            MSXExxxx.TransducerTypeInfo[] transducer_info = new Configuration.MSXExxxx.TransducerTypeInfo[16];
            MSXExxxx.DefaultResponse response;

            try
            {
                response = msxe_soap.TransducerGetChannelTransducersList(channel, 0, out transducer_nb, out transducer_calibrated, out transducer_info);
                if (response.iReturnValue != 0)
                {
                    MessageBox.Show("TransducerGetChannelTransducersList error : " + response.syserrno);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("TransducerGetChannelTransducersList exception : " + ex.Message);
                return;
            }

            // Construct data source
            datasource[channel] = new string[transducer_nb];
            transducer_indexes[channel] = new ulong[transducer_nb];
            for (i = 0; i < transducer_nb; i++)
            {
                string type = "HB";
                if ( transducer_info[i].ulType == 1 )
                {
                    type = "LVDT";
                }

                // Set the string
                datasource[channel][i] = transducer_info[i].ulTransducerIndex + " - " + type + " - " + get_name(transducer_info[i].cName);

                // Save the index too
                transducer_indexes[channel][i] = transducer_info[i].ulTransducerIndex;
            }
        }

        /// <summary>
        /// Returns the index of the variable value in array
        /// </summary>
        /// <param name="array"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private int get_index_in_array(ulong[] array, ulong value)
        {
            int i = 0;
            for (i = 0; i < array.Length; i++)
            {
                if (array[i] == value)
                    return i;
            }
            return 0;
        }

        /// <summary>
        /// Occurs when the user clicks on connection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connection_button_Click(object sender, EventArgs e)
        {
            ulong i = 0;
            MSXExxxx.DefaultResponse response;
            ulong[] channels_type = new ulong[8];
            int current_y = 5;

            // Clear controls
            information_panel.Controls.Clear();

            // Check the ip
            if (NetworkHelper.isIpCorrect(ip_field_0.Text, ip_field_1.Text, ip_field_2.Text, ip_field_3.Text) == false)
            {
                MessageBox.Show("The given ip address is wrong. Please correct it and retry");
                return;
            }

            // Init soap connection
            msxe_soap.Url = "http://" + ip_field_0.Text + "." + ip_field_1.Text + "." + ip_field_2.Text + "." + ip_field_3.Text + ":5555";
            msxe_soap.Timeout = 3000;

            // Get the channel number
            try
            {
                response = msxe_soap.TransducerGetNumberOfChannels(0, out channel_nb);
                if (response.iReturnValue != 0)
                {
                    MessageBox.Show("TransducerGetNumberOfChannels error : " + response.syserrno);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("TransducerGetNumberOfChannels exception : " + ex.Message);
                return;
            }

            // Create the graphics
            channel_id = new Label[channel_nb];
            type = new Label[channel_nb];
            invert_value = new RadioButton[channel_nb];
            dont_invert_value = new RadioButton[channel_nb];
            calibrated = new CheckBox[channel_nb];
            transducer = new ComboBox[channel_nb];
            invert_or_not = new Panel[channel_nb];
            datasource = new string[channel_nb][];
            transducer_indexes = new ulong[channel_nb][];

            // Get type information
            try
            {
                response = msxe_soap.TransducerGetChannelsType(0, out channels_type);
                if (response.iReturnValue != 0)
                {
                    MessageBox.Show("TransducerGetChannelsType syserrno : " + response.syserrno);
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("TransducerGetChannelsType exception : " + ex.Message);
                return;
            }

            // Construct info's
            for (i = 0; i < channel_nb; i++)
            {
                ulong current_type = 0;
                ulong current_transducer_index = 0;
                ulong current_calibrated = 0;
                ulong current_direction = 0;

                // Get channel configuration
                try
                {
                    response = msxe_soap.TransducerGetChannelConfiguration(i, 0, out current_type, out current_transducer_index, out current_calibrated, out current_direction);
                    if (response.iReturnValue != 0)
                    {
                        MessageBox.Show("TransducerGetChannelConfiguration syserrno : " + response.syserrno);
                        return;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("TransducerGetChannelConfiguration exception : " + ex.Message);
                    return;
                }

                // Construct graphics

                // Channel ID
                channel_id[i] = new Label();
                channel_id[i].Text = i.ToString();
                channel_id[i].SetBounds(5, current_y, 60, 17);
                information_panel.Controls.Add(channel_id[i]);

                // Type
                type[i] = new Label();
                if (current_type == 0)
                {
                    type[i].Text = "HB";
                }
                else
                {
                    type[i].Text = "LVDT";
                }
                type[i].SetBounds(80, current_y, 60, 17);
                information_panel.Controls.Add(type[i]);

                // Invert / Don't invert value
                invert_or_not[i] = new Panel();
                invert_or_not[i].SetBounds(145, current_y - 2, 145, 17);

                invert_value[i] = new RadioButton();
                invert_value[i].Text = "invert";
                invert_value[i].SetBounds(0, 0, 60, 17);
                invert_or_not[i].Controls.Add(invert_value[i]);

                dont_invert_value[i] = new RadioButton();
                dont_invert_value[i].Text = "don't invert";
                dont_invert_value[i].SetBounds(65,0, 80, 17);
                invert_or_not[i].Controls.Add(dont_invert_value[i]);

                information_panel.Controls.Add(invert_or_not[i]);

                if (current_direction == 1)
                {
                    invert_value[i].Checked = true;
                }
                else
                {
                    dont_invert_value[i].Checked = true;
                }

                // Transducer
                get_transducer_information(i); // prepare data source
                transducer[i] = new ComboBox();
                transducer[i].DataSource = datasource[i];
                transducer[i].SetBounds(300, current_y-2, 200, 17);
                information_panel.Controls.Add(transducer[i]);
                information_panel.Update();

                // Set selected transducer
                transducer[i].SelectedIndex = get_index_in_array(transducer_indexes[i], current_transducer_index);

                // Calibration
                calibrated[i] = new CheckBox();
                calibrated[i].SetBounds(520, current_y, 50, 17);
                calibrated[i].Enabled = false;
                information_panel.Controls.Add(calibrated[i]);
                if (current_calibrated == 1)
                {
                    calibrated[i].Checked = true;
                }

                current_y += 25;
            }

            // Enables to save the configuration
            save_configuration.Enabled = true;
        }

        /// <summary>
        /// The user clicks on save
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void save_configuration_Click(object sender, EventArgs e)
        {
            ulong i = 0;
            for (i = 0; i < channel_nb; i++)
            {
                ulong current_direction = 0;
                int syserrno = 0;
                if (invert_value[i].Checked == true)
                {
                    current_direction = 1;
                }
                // Save the channel configuration
                try
                {
                    if (msxe_soap.TransducerSetChannelConfiguration(i, transducer_indexes[i][transducer[i].SelectedIndex], current_direction, 0, out syserrno) != 0)
                    {
                        MessageBox.Show("TransducerSetChannelConfiguration syserrno : " + syserrno );
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("TransducerSetChannelConfiguration exception : " + ex.Message);
                    return;
                }
            }
            MessageBox.Show("Configuration saved");
        }
    }
}