using System;
using System.Collections.Generic;
using System.Text;

namespace TransducerConfiguration
{
    public class MSXExxxx
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXExxxx(String ip)
        {
            // Construct the web reference
            this.sys = new MSXExxxxWeb.MSXExxxx();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 5000;
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the number of transducer channels
        /// </summary>
        /// <param name="channelNb"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerGetNumberOfChannels(ref ulong channelNb, ref string error)
        {
            // init
            error = string.Empty;
            channelNb = 0;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.TransducerGetNumberOfChannels(0, out channelNb);
                if (response.iReturnValue != 0)
                {
                    error = "TransducerGetNumberOfChannels error: " + response.iReturnValue.ToString();
                    return -2;
                }
            }
            catch (Exception ex)
            {
                error = "TransducerGetNumberOfChannels exception: " + ex.Message;
                return -3;
            }

            return 0;
        }

        /// <summary>
        /// Get informations on the available transducers for a given channel
        /// </summary>
        /// <param name="channel"></param>
        /// <param name="transducers"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerGetChannelTransducersList(ulong channel, ref List<Transducer> transducers, ref string error)
        {
            // init
            error = string.Empty;
            
            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            ulong transducerNb = 0;
            ulong [] calibrated = null;
            MSXExxxxWeb.TransducerTypeInfo [] info = null;

            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.TransducerGetChannelTransducersList(channel,
                0,
                out transducerNb,
                out calibrated,
                out info);
                if (response.iReturnValue != 0)
                {
                    error = "TransducerGetChannelTransducersList error: " + response.iReturnValue.ToString();
                    return -2;
                }

                // Construct the outside list
                transducers = new List<Transducer>();
                for (ulong i = 0; i < transducerNb; i++)
                {
                    transducers.Add(new Transducer(info[i].ulTransducerIndex,
                        new ASCIIEncoding().GetString(info[i].cName),
                        info[i].ulType,
                        info[i].dVeff,
                        info[i].dRange,
                        info[i].dSensitivity,
                        info[i].ulNominalFrequency,
                        info[i].ulImpedance,
                        calibrated[i]));
                }
            }
            catch (Exception ex)
            {
                error = "TransducerGetChannelTransducersList exception: " + ex.Message;
                return -3;
            }

            return 0;
        }
    }
}
